#define STICKY_NOTE_INPUT_C

#include "main.h"

static int sock_fd = -1;
static struct sockaddr_un sa;

static const char *note_name = NULL;

static void init_cmd_line_options(int argc, char *argv[])
{
 const char *specified_note_dir = NULL;
 struct option long_options[] =
 { 
  {"sticky-create", no_argument, NULL, 'c'},
  {"sticky-name", required_argument, NULL, 'n'},
  {"sticky-dir", required_argument, NULL, 'd'},
  {"help", no_argument, NULL, 'h'},
  {"version", no_argument, NULL, 'v'},  
  {NULL, 0, NULL, 0}
 };
 const char *program_name = argv[0];
 int optchar = 0;
 
 if ((program_name = strrchr(argv[0], '/')))
   ++program_name;
 else
   program_name = argv[0];
 
 while ((optchar = getopt_long(argc, argv, "HhVvcd:n:", long_options, NULL))
        != EOF)
   switch (optchar)
   {
    case '?':
      fprintf(stderr, "%s", " Unknown option...\n");
    case 'H':
    case 'h':
      fprintf(stderr, " Usage: %s [-dhv]\n%s", program_name,
              " --sticky-create -c     - Send the input to a new note (default).\n"
              " --sticky-dir -d        - Change the base directory.\n"
              " --help -h              - Print this message.\n"
              " --sticky-name -n       - Send the input to a specific note.\n"
              " --version -v           - Print the version.\n");
      exit (optchar == 'h' ? EXIT_SUCCESS : EXIT_FAILURE);

    case 'V':
    case 'v':
      printf("%s", " Version: 0.8 of sticky-notes, by James Antill\n");
      exit (EXIT_SUCCESS);

    case 'd':
      specified_note_dir = optarg;
      break;

    case 'n':
      note_name = optarg;
      break;

    case 'c':
      note_name = NULL;
      break;
      
    default:
      assert(FALSE);
   }
 
 if (specified_note_dir)
 {
  if (chdir(specified_note_dir))
  {
   fprintf(stderr, " Could not cd to (or create) specified directory.\n");
   exit (EXIT_FAILURE);
  }
 }
 else
 {
  if (chdir(getenv("HOME")))
  {
   fprintf(stderr, " Could not cd to home directory.\n");
   exit (EXIT_FAILURE);
  }
  
  if (chdir(".sticky"))
  {
   fprintf(stderr, " Could not cd to (or create) .sticky directory in "
           "your home directory.\n");
   exit (EXIT_FAILURE);
  }
 }
}

static void send_data_server(const char *buffer, size_t length)
{
 size_t partial = 0;

 assert(length);
 
 if (length)
   while (partial < length)
   {
    ssize_t tmp = write(sock_fd, buffer + partial, length - partial);
    
    if (tmp != -1)
      partial += (size_t) tmp;
    else
      switch (errno)
      {
       case EINTR: /* interupt occured */
       case EAGAIN: /* too much data */
       case ENOSPC: /* too much data */
       {
        struct pollfd fds[1];
        
        fds[0].fd = sock_fd;
        fds[0].events = POLLOUT;
        
        poll(fds, 1, 1000); /* one second */
       }
       continue;
         
       case EPIPE: /* sigpipe */
       default:
         close(sock_fd);
         fprintf(stderr, " Failed to write data to sticky note server.\n");
         exit (EXIT_FAILURE);
      }
   }
}

static void send_header(void)
{
 struct pollfd fds[1];
 char answer;
 
 if (note_name)
 {
  size_t len = strlen(note_name);
  
#ifdef USE_LESS_WRITES
  char *tmp = malloc(len + 1);
  
  if (!tmp)
  {
   close(sock_fd);
   fprintf(stderr, " Failed to allocate memory for note name.\n");
   exit (EXIT_FAILURE);
  }
#endif
  
  send_data_server("note_name:", CONST_STRLEN("note_name:"));
  
#ifdef USE_LESS_WRITES
  memcpy(tmp, note_name, len);
  tmp[len] = '\n';

  send_data_server(tmp, len + 1);
#else
  send_data_server(note_name, len);
  send_data_server("\n", 1);
#endif
 }
 else
   send_data_server("new_note\n", CONST_STRLEN("new_note\n"));

 fds[0].fd = sock_fd;
 fds[0].events = POLLIN;

 poll(fds, 1, -1);

 while (read(sock_fd, &answer, 1) != 1) { /* loop */ }

 if (!answer)
 {
  close(sock_fd);
  if (note_name)
    fprintf(stderr, " There does not exist a note of the name -- %s.\n",
            note_name);
  else
    fprintf(stderr, " Invalid header.\n");
  exit (EXIT_FAILURE);
 }
}

static void loop_data_through(void)
{
 do
 {
  char buffer[1024];
  size_t read_data = fread(buffer, sizeof(char), 1024, stdin);

  send_data_server(buffer, read_data);
  
 } while (!feof(stdin));
}

int main(int argc, char *argv[])
{
 init_cmd_line_options(argc, argv);
 
 if ((sock_fd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
 {
  fprintf(stderr, " Failed to connect to sticky notes server.\n");
  exit (EXIT_FAILURE);
 }
  
 sa.sun_family = AF_UNIX;
 strcpy(sa.sun_path, "coms");
  
 if (connect(sock_fd, (struct sockaddr *)&sa, sizeof(sa)) == -1)
 {
  close(sock_fd);
  fprintf(stderr, " Failed to connect to sticky notes server.\n");
  exit (EXIT_FAILURE);
 }

 send_header();
 
 loop_data_through();

 exit (EXIT_SUCCESS);
}
